

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `Frame': Representation of a general array entity.            //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

struct Frame {

  // Constructors

  Frame(const int _rank, const Group& _group) : rank(_rank), group(_group) {}

  Frame(const int _rank, const Group& _group,
        Range* _ranges, Stride* _strides) :
            rank(_rank), group(_group), ranges(_ranges), strides(_strides) {}

  // Data members

  const int rank ;

  Group group ;

  Range*  ranges  ;
  Stride* strides ;

  // inquiries

  int rnk() const {return rank ; }

  const Group &grp() const {return group ; }

  const Range  rng(const int r) const {return ranges  [r] ; }
  const Stride str(const int r) const {return strides [r] ; }

  DimensionSet sig() const {
    DimensionSet signature ;
    for(int r = 0 ; r < rank ; r++)
      signature += ranges [r].dim() ;
    return signature ;
  }
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// `shapeCheck'                                                        //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

inline void shapeCheck(const Frame* a, const Frame* b) {
  if(a->rnk() != b->rnk())
    AdlibError("shapeCheck: Rank mismatch") ;
  for(int i = 0 ; i < a->rnk() ; i++)
    if(a->rng(i).size() != b->rng(i).size())
      AdlibError("shapeCheck: Shape mismatch") ;
}


// ----------------------------------------------------------------------
// Things to do
//
//      Make `Frame' a subclass of `Group'???  Not sure why, but it
//      seems like an obvious thought.  Probably not.

